"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseWhereKey = exports.applyWhere = void 0;
var _ElasticsearchFieldPlugin = require("./plugins/definition/ElasticsearchFieldPlugin");
var _error = _interopRequireDefault(require("@webiny/error"));
/**
 * TODO remove the wbyAco prefix when we move the user fields to the values property.
 */
const parseWhereKeyRegExp = new RegExp(/^((?:wbyAco_)?[a-zA-Z0-9]+)(_[a-zA-Z0-9_]+)?$/);
const parseWhereKey = key => {
  const match = key.match(parseWhereKeyRegExp);
  if (!match) {
    throw new Error(`It is not possible to search by key "${key}"`);
  }
  const [, field, operation = "eq"] = match;
  if (!field.match(/^(?:wbyAco_)?([a-zA-Z0-9]+)$/)) {
    throw new Error(`Cannot filter by "${field}".`);
  }
  const operator = operation.match(/^_/) ? operation.slice(1) : operation;
  return {
    field,
    operator
  };
};
exports.parseWhereKey = parseWhereKey;
const ALL = _ElasticsearchFieldPlugin.ElasticsearchFieldPlugin.ALL;
const applyWhere = params => {
  const {
    query,
    where,
    fields,
    operators
  } = params;
  for (const key in where) {
    if (where.hasOwnProperty(key) === false) {
      continue;
    }
    const initialValue = where[key];
    /**
     * There is a possibility that undefined is sent as a value, so just skip it.
     */
    if (initialValue === undefined) {
      continue;
    }
    const {
      field,
      operator
    } = parseWhereKey(key);
    const fieldPlugin = fields[field] || fields[ALL];
    if (!fieldPlugin) {
      throw new _error.default(`Missing plugin for the field "${field}".`, "PLUGIN_WHERE_ERROR", {
        field
      });
    }
    const operatorPlugin = operators[operator];
    if (!operatorPlugin) {
      throw new _error.default(`Missing plugin for the operator "${operator}"`, "PLUGIN_WHERE_ERROR", {
        operator
      });
    }

    /**
     * Get the path but in the case of * (all fields, replace * with the field.
     * Custom path would return its own value anyways.
     */
    const path = fieldPlugin.getPath(field);
    const basePath = fieldPlugin.getBasePath(field);
    /**
     * Transform the value for the search.
     */
    const value = fieldPlugin.toSearchValue({
      value: initialValue,
      path,
      basePath
    });
    operatorPlugin.apply(query, {
      name: field,
      value,
      path,
      basePath,
      keyword: fieldPlugin.keyword
    });
  }
};
exports.applyWhere = applyWhere;

//# sourceMappingURL=where.js.map